# NOTE: this class has been translated from Java.
# Do not modify this file, changes will be overwritten.

#package orbitgt.mapping3d.sdk.viewer;

from typing import Generic,TypeVar
T = TypeVar('T')

from .....com.orbitgis.lib.util.ArrayList import ArrayList
from .....com.orbitgis.tool.Strings import Strings
from .....orbitgt.mapping3d.sdk.viewer.IList import IList

#
# Class AList defines a list of typed objects.
#

class AList(Generic[T],IList):
    # The generic type */
    #private String genericType
    # The elements */
    #private ArrayList[T] elements

    #
    # Create a new list.
    # @param genericType the generic type of the elements in the list, like MapObject.TYPE for example.
    #
    def __init__(self,genericType: str):
        self.genericType = genericType
        self.elements = ArrayList[T]()

    #
    # Get the generic type of the elements.
    # @return the generic type of the elements, like MapObject.TYPE for example.
    #
    def getGenericType(self) -> str:
        return self.genericType

    #
    # Add an element.
    # @param element the element to add.
    #
    def add(self,element: T) -> None:
        self.elements.add(element)

    #
    # Add an untyped object.
    # @param element the element to add.
    #
    def addObject(self,element: object) -> None:
        self.add(element)

    #
    # Add a list of elements.
    # @param list the list.
    #
    def addList(self,list: 'AList[T]') -> None:
        if (list==None): return
        for i in range(0,list.size()): self.add(list.get(i))

    #
    # Add a list of untyped objects.
    # @param list the list.
    #
    def addObjectList(self,list: 'AList[object]') -> None:
        if (list==None): return
        for i in range(0,list.size()): self.addObject(list.get(i))

    #
    # Add an element at a certain index.
    # @param index the index to add at.
    # @param element the element to add.
    #
    def addAt(self,index: int,element: T) -> None:
        self.elements.add(index,element)

    #
    # Remove an element.
    # @param index the index to remove at.
    # @return the removed element.
    #
    def removeAt(self,index: int) -> T:
        element = self.elements.get(index)
        self.elements.remove(index)
        return element

    #
    # Clear the list.
    #
    def clear(self) -> None:
        self.elements.clear()

    #
    # Get the number of elements.
    # @return the number of elements.
    #
    def size(self) -> int:
        return self.elements.size()

    #
    # Get an element.
    # @param index the index of the element.
    # @return the element.
    #
    def get(self,index: int) -> T:
        return self.elements.get(index)

    #
    # Get an untyped object.
    # @param index the index of the object.
    # @return the element.
    #
    def getObject(self,index: int) -> object:
        return self.elements.get(index)

    #
    # Set an element.
    # @param index the index of the element.
    # @param element the element.
    #
    def set(self,index: int,element: T) -> None:
        self.elements.set(index,element)

    #
    # Set an untyped element.
    # @param index the index of the element.
    # @param element the element.
    #
    def setObject(self,index: int,element: object) -> None:
        self.elements.set(index,element)

    #
    # Convert the object to a string.
    # @return the string form of the object.
    #
    def toString(self) -> str:
        return "[AList:type="+self.genericType+",size="+Strings.toString(self.size())+"]"

    # csharp add begin
    #    /**
    #     * Get a native array.
    #     * @return a native array of type T[].
    #     */
    #    public T[] toArray()
    #    {
    #    	T[] array = new T[this.size()];
    #    	for(int i=0; i<array.Length; i++) array[i] = this.get(i);
    #    	return array;
    #    }
    # csharp add end

    # haxe add begin
    #    #if js
    #    /**
    #     * Create a list from a native array.
    #     * @param list -
    #     * @return AList<T>
    #     */
    #    public static function fromArray<T2>(list:Array<T2>,genericType:String):AList<T2>
    #    {
    #        var instance = new AList<T2>(genericType);
    #        for (item in list) instance.add(item);
    #        return instance;
    #    }
    #    /**
    #     * Get a native array.
    #     * @return a native array.
    #     */
    #    public function toArray(): Array<T>
    #    {
    #        return this.elements.getElementData();
    #    }
    #    #end
    # haxe add end

    # typescript add begin
    #    /**
    #     * Create a list from a native array.
    #     * @param list -
    #     * @return AList<T>
    #     */
    #    public static fromArray<T2>(list:Array<T2>,genericType:string):AList<T2>
    #    {
    #        let instance: AList<T2> = new AList<T2>(genericType);
    #        for (let item of list) instance.add(item);
    #        return instance;
    #    }
    #    /**
    #     * Get a native array.
    #     * @return a native array.
    #     */
    #    public toArray(): Array<T>
    #    {
    #        return this.elements.getArray();
    #    }
    # typescript add end
